@@ -2,6 +2,7 @@ from wtforms import fields, validators
 from sqlalchemy import Boolean, Column
 
 from flask.ext.admin import form
+from flask.ext.admin.form import Select2Field
 from flask.ext.admin.tools import get_property
 from flask.ext.admin.model.form import (converts, ModelConverterBase,
                                         InlineFormAdmin, InlineModelConverterBase)
@@ -78,7 +79,7 @@ class AdminModelConverter(ModelConverterBase):
             if local_column.nullable:
                 kwargs['validators'].append(validators.Optional())
             elif prop.direction.name != 'MANYTOMANY':
-                kwargs['validators'].append(validators.Required())
+                kwargs['validators'].append(validators.InputRequired())
 
             # Override field type if necessary
             override = self._get_field_override(prop.key)
@@ -153,7 +154,7 @@ class AdminModelConverter(ModelConverterBase):
                                                        column))
 
                 if not column.nullable and not isinstance(column.type, Boolean):
-                    kwargs['validators'].append(validators.Required())
+                    kwargs['validators'].append(validators.InputRequired())
 
                 # Apply label and description if it isn't inline form field
                 if self.view.model == mapper.class_:
@@ -169,7 +170,7 @@ class AdminModelConverter(ModelConverterBase):
 
                     if value is not None:
                         if getattr(default, 'is_callable', False):
-                            value = value(None)
+                            value = lambda: default.arg(None)
                         else:
                             if not getattr(default, 'is_scalar', True):
                                 value = None
@@ -186,6 +187,16 @@ class AdminModelConverter(ModelConverterBase):
                 if override:
                     return override(**kwargs)
 
+                # Check choices
+                if mapper.class_ == self.view.model and self.view.form_choices:
+                    choices = self.view.form_choices.get(column.key)
+                    if choices:
+                        return Select2Field(
+                            choices=choices,
+                            allow_blank=column.nullable,
+                            **kwargs
+                        )
+
                 # Run converter
                 converter = self.get_converter(column)
 
@@ -277,6 +288,20 @@ class AdminModelConverter(ModelConverterBase):
         return form.Select2TagsField(save_as_list=True, **field_args)
 
 
+def _resolve_prop(prop):
+    """
+        Resolve proxied property
+
+        :param prop:
+            Property to resolve
+    """
+    # Try to see if it is proxied property
+    if hasattr(prop, '_proxied_property'):
+        return prop._proxied_property
+
+    return prop
+
+
 # Get list of fields and generate form
 def get_form(model, converter,
             base_class=form.BaseForm,
@@ -320,11 +345,8 @@ def get_form(model, converter,
         def find(name):
             # Try to look it up in properties list first
             p = props.get(name)
-            if p is not None:
-                # Try to see if it is proxied property
-                if hasattr(p, '_proxied_property'):
-                    return p._proxied_property
 
+            if p is not None:
                 return p
 
             # If it is hybrid property or alias, look it up in a model itself
@@ -340,11 +362,13 @@ def get_form(model, converter,
         properties = (x for x in properties if x[0] not in exclude)
 
     field_dict = {}
-    for name, prop in properties:
+    for name, p in properties:
         # Ignore protected properties
         if ignore_hidden and name.startswith('_'):
             continue
 
+        prop = _resolve_prop(p)
+
         field = converter.convert(model, mapper, prop, field_args.get(name), hidden_pk)
         if field is not None:
             field_dict[name] = field
@@ -419,7 +443,7 @@ class InlineModelConverter(InlineModelConverterBase):
 
         for prop in target_mapper.iterate_properties:
             if hasattr(prop, 'direction') and prop.direction.name == 'MANYTOONE':
-                if prop.mapper.class_ == model:
+                if issubclass(model, prop.mapper.class_):
                     reverse_prop = prop
                     break
         else:
@@ -467,7 +491,7 @@ class InlineModelConverter(InlineModelConverterBase):
                 InlineModelFormList(child_form,
                                     self.session,
                                     info.model,
-                                    forward_prop.key,
+                                    reverse_prop.key,
                                     **kwargs))
 
         return form_class
