@@ -18,7 +18,7 @@ from .typefmt import DEFAULT_FORMATTERS
 
 class ModelView(BaseModelView):
     """
-        SQLALchemy model view
+        SQLAlchemy model view
 
         Usage sample::
 
@@ -90,7 +90,7 @@ class ModelView(BaseModelView):
             class MyModelView(ModelView):
                 column_searchable_list = (User.name, User.email)
 
-        Following search rules apply:
+        The following search rules apply:
 
         - If you enter *ZZZ* in the UI search field, it will generate *ILIKE '%ZZZ%'*
           statement against searchable columns.
@@ -103,8 +103,8 @@ class ModelView(BaseModelView):
         - If you prefix your search term with ^, it will find all rows
           that start with ^. So, if you entered *^ZZZ*, *ILIKE 'ZZZ%'* will be used.
 
-        - If you prefix your search term with =, it will do exact match.
-          For example, if you entered *=ZZZ*, *ILIKE 'ZZZ'* statement will be used.
+        - If you prefix your search term with =, it will perform an exact match.
+          For example, if you entered *=ZZZ*, the statement *ILIKE 'ZZZ'* will be used.
     """
 
     column_filters = None
@@ -163,11 +163,11 @@ class ModelView(BaseModelView):
     """
         If set to `False` and user deletes more than one model using built in action,
         all models will be read from the database and then deleted one by one
-        giving SQLAlchemy chance to manually cleanup any dependencies (many-to-many
+        giving SQLAlchemy a chance to manually cleanup any dependencies (many-to-many
         relationships, etc).
 
-        If set to `True`, will run `DELETE` statement which is somewhat faster,
-        but might leave corrupted data if you forget to configure `DELETE
+        If set to `True`, will run a `DELETE` statement which is somewhat faster,
+        but may leave corrupted data if you forget to configure `DELETE
         CASCADE` for your model.
     """
 
@@ -195,9 +195,9 @@ class ModelView(BaseModelView):
             class MyModelView(ModelView):
                 inline_models = (MyInlineModelForm(MyInlineModel),)
 
-        You can customize generated field name by:
+        You can customize the generated field name by:
 
-        1. Using `form_name` property as option:
+        1. Using the `form_name` property as a key to the options dictionary:
 
             class MyModelView(ModelView):
                 inline_models = ((Post, dict(form_label='Hello')))
@@ -218,6 +218,18 @@ class ModelView(BaseModelView):
 
     column_type_formatters = DEFAULT_FORMATTERS
 
+    form_choices = None
+    """
+        Map choices to form fields
+
+        Example::
+
+            class MyModelView(BaseModelView):
+                form_choices = {'my_form_field': [
+                    ('db_value', 'display_value'),
+                ]
+    """
+
     def __init__(self, model, session,
                  name=None, category=None, endpoint=None, url=None):
         """
@@ -226,15 +238,15 @@ class ModelView(BaseModelView):
             :param model:
                 Model class
             :param session:
-                SQLALchemy session
+                SQLAlchemy session
             :param name:
-                View name. If not set, will default to model name
+                View name. If not set, defaults to the model name
             :param category:
                 Category name
             :param endpoint:
-                Endpoint name. If not set, will default to model name
+                Endpoint name. If not set, defaults to the model name
             :param url:
-                Base URL. If not set, will default to '/admin/' + endpoint
+                Base URL. If not set, defaults to '/admin/' + endpoint
         """
         self.session = session
 
@@ -243,6 +255,9 @@ class ModelView(BaseModelView):
 
         self._filter_joins = dict()
 
+        if self.form_choices is None:
+            self.form_choices = {}
+
         super(ModelView, self).__init__(model, name, category, endpoint, url)
 
         # Primary key
@@ -270,19 +285,19 @@ class ModelView(BaseModelView):
     # Scaffolding
     def scaffold_pk(self):
         """
-            Return primary key name from a model
+            Return the primary key name from a model
         """
         return tools.get_primary_key(self.model)
 
     def get_pk_value(self, model):
         """
-            Return PK value from a model object.
+            Return the PK value from a model object.
         """
         return getattr(model, self._primary_key)
 
     def scaffold_list_columns(self):
         """
-            Return list of columns from the model.
+            Return a list of columns from the model.
         """
         columns = []
 
@@ -307,7 +322,7 @@ class ModelView(BaseModelView):
 
     def scaffold_sortable_columns(self):
         """
-            Return dictionary of sortable columns.
+            Return a dictionary of sortable columns.
             Key is column name, value is sort column/field.
         """
         columns = dict()
@@ -316,13 +331,12 @@ class ModelView(BaseModelView):
             if hasattr(p, 'columns'):
                 # Sanity check
                 if len(p.columns) > 1:
-                    raise Exception('Automatic form scaffolding is not supported' +
-                                    ' for multi-column properties (%s.%s)' % (
-                                                    self.model.__name__, p.key))
+                    # Multi-column properties are not supported
+                    continue
 
                 column = p.columns[0]
 
-                # Can't sort by on primary and foreign keys by default
+                # Can't sort on primary or foreign keys by default
                 if column.foreign_keys:
                     continue
 
@@ -383,7 +397,7 @@ class ModelView(BaseModelView):
 
     def is_text_column_type(self, name):
         """
-            Verify if column type is text-based.
+            Verify if the provided column type is text-based.
 
             :returns:
                 ``True`` for ``String``, ``Unicode``, ``Text``, ``UnicodeText``
@@ -394,8 +408,22 @@ class ModelView(BaseModelView):
         """
             Return list of enabled filters
         """
+
+        join_tables = []
         if isinstance(name, basestring):
-            attr = getattr(self.model, name, None)
+            model = self.model
+
+            for attribute in name.split('.'):
+                value = getattr(model, attribute)
+                if (hasattr(value, 'property') and
+                    hasattr(value.property, 'direction')):
+                    model = value.property.mapper.class_
+                    table = model.__table__
+
+                    if self._need_join(table):
+                        join_tables.append(table)
+
+                attr = value
         else:
             attr = name
 
@@ -423,9 +451,12 @@ class ModelView(BaseModelView):
                                                         visible_name)
 
                     if flt:
-                        if self._need_join(column.table):
-                            self._filter_joins[column.table.name] = column.table
+                        table = column.table
 
+                        if join_tables:
+                            self._filter_joins[table.name] = join_tables
+                        elif self._need_join(table.name):
+                            self._filter_joins[table.name] = [table.name]
                         filters.extend(flt)
 
             return filters
@@ -437,26 +468,37 @@ class ModelView(BaseModelView):
 
             column = columns[0]
 
-            if not isinstance(name, basestring):
-                visible_name = self.get_column_name(name.property.key)
+            if self._need_join(column.table) and name not in self.column_labels:
+                visible_name = '%s / %s' % (
+                    self.get_column_name(column.table.name),
+                    self.get_column_name(column.name)
+                )
             else:
-                visible_name = self.get_column_name(name)
+                if not isinstance(name, basestring):
+                    visible_name = self.get_column_name(name.property.key)
+                else:
+                    visible_name = self.get_column_name(name)
 
             type_name = type(column.type).__name__
-            flt = self.filter_converter.convert(type_name,
-                                                column,
-                                                visible_name)
 
-            if flt:
-                # If there's relation to other table, do it
-                if self._need_join(column.table):
-                    self._filter_joins[column.table.name] = column.table
+            if join_tables:
+                self._filter_joins[column.table.name] = join_tables
+
+            flt = self.filter_converter.convert(
+                type_name,
+                column,
+                visible_name,
+                options=self.column_choices.get(name),
+            )
+
+            if flt and not join_tables and self._need_join(column.table):
+                self._filter_joins[column.table.name] = [column.table]
 
             return flt
 
     def is_valid_filter(self, filter):
         """
-            Verify that provided filter object is derived from the
+            Verify that the provided filter object is derived from the
             SQLAlchemy-compatible filter class.
 
             :param filter:
@@ -499,7 +541,7 @@ class ModelView(BaseModelView):
 
     def scaffold_auto_joins(self):
         """
-            Return list of joined tables by going through the
+            Return a list of joined tables by going through the
             displayed columns.
         """
         if not self.column_auto_select_related:
@@ -579,12 +621,15 @@ class ModelView(BaseModelView):
             for idx, value in filters:
                 flt = self._filters[idx]
 
-                # Figure out join
+                # Figure out joins
                 tbl = flt.column.table.name
-                join = self._filter_joins.get(tbl)
-                if join is not None:
-                    query = query.join(join)
-                    joins.add(tbl)
+
+                join_tables = self._filter_joins.get(tbl, [])
+
+                for table in join_tables:
+                    if table.name not in joins:
+                        query = query.join(table)
+                        joins.add(table)
 
                 # Apply filter
                 query = flt.apply(query, value)
@@ -601,6 +646,7 @@ class ModelView(BaseModelView):
             if sort_column in self._sortable_columns:
                 sort_field = self._sortable_columns[sort_column]
 
+                # TODO: Preprocessing for joins
                 # Try to handle it as a string
                 if isinstance(sort_field, basestring):
                     # Create automatic join against a table if column name
@@ -612,11 +658,18 @@ class ModelView(BaseModelView):
                             query = query.join(parts[0])
                             joins.add(parts[0])
                 elif isinstance(sort_field, InstrumentedAttribute):
-                    table = sort_field.parententity.tables[0]
+                    # SQLAlchemy 0.8+ uses 'parent' as a name
+                    mapper = getattr(sort_field, 'parent', None)
+                    if mapper is None:
+                        # SQLAlchemy 0.7.x uses parententity
+                        mapper = getattr(sort_field, 'parententity', None)
 
-                    if table.name not in joins:
-                        query = query.join(table)
-                        joins.add(table.name)
+                    if mapper is not None:
+                        table = mapper.tables[0]
+
+                        if table.name not in joins:
+                            query = query.join(table)
+                            joins.add(table.name)
                 elif isinstance(sort_field, Column):
                     pass
                 else:
@@ -642,7 +695,7 @@ class ModelView(BaseModelView):
 
     def get_one(self, id):
         """
-            Return one model by its id.
+            Return a single model by its id.
 
             :param id:
                 Model id
