@@ -38,10 +38,10 @@ def list_formatter(values):
         :param values:
             Value to check
     """
-    return u', '.join(values)
+    return u', '.join(unicode(v) for v in values)
 
 
-DEFAULT_FORMATTERS = {
+BASE_FORMATTERS = {
     type(None): empty_formatter,
     bool: bool_formatter,
     list: list_formatter,
