@@ -20,22 +20,37 @@ class CustomModelView(ModelView):
 
 def create_models(db):
     class Model1(db.Model):
-        def __init__(self, test1=None, test2=None, test3=None, test4=None):
+        def __init__(self, test1=None, test2=None, test3=None, test4=None, bool_field=False):
             self.test1 = test1
             self.test2 = test2
             self.test3 = test3
             self.test4 = test4
+            self.bool_field = bool_field
 
         id = db.Column(db.Integer, primary_key=True)
         test1 = db.Column(db.String(20))
         test2 = db.Column(db.Unicode(20))
         test3 = db.Column(db.Text)
         test4 = db.Column(db.UnicodeText)
+        bool_field = db.Column(db.Boolean)
+        enum_field = db.Column(db.Enum('model1_v1', 'model1_v1'), nullable=True)
 
     class Model2(db.Model):
+        def __init__(self, string_field=None, int_field=None, bool_field=None, model1=None):
+            self.string_field = string_field
+            self.int_field = int_field
+            self.bool_field = bool_field
+            self.model1 = model1
+
         id = db.Column(db.Integer, primary_key=True)
+        string_field = db.Column(db.String)
         int_field = db.Column(db.Integer)
         bool_field = db.Column(db.Boolean)
+        enum_field = db.Column(db.Enum('model2_v1', 'model2_v2'), nullable=True)
+
+        # Relation
+        model1_id = db.Column(db.Integer, db.ForeignKey(Model1.id))
+        model1 = db.relationship(Model1)
 
     db.create_all()
 
@@ -151,11 +166,16 @@ def test_exclude_columns():
 
     Model1, Model2 = create_models(db)
 
-    view = CustomModelView(Model1, db.session,
-                           column_exclude_list=['test2', 'test4'])
+    view = CustomModelView(
+        Model1, db.session,
+       column_exclude_list=['test2', 'test4', 'enum_field']
+    )
     admin.add_view(view)
 
-    eq_(view._list_columns, [('test1', 'Test1'), ('test3', 'Test3')])
+    eq_(
+        view._list_columns,
+        [('test1', 'Test1'), ('test3', 'Test3'), ('bool_field', 'Bool Field')]
+    )
 
     client = app.test_client()
 
@@ -196,34 +216,99 @@ def test_column_filters():
 
     Model1, Model2 = create_models(db)
 
-    view = CustomModelView(Model1, db.session,
-                           column_filters=['test1'])
+    view = CustomModelView(
+        Model1, db.session,
+        column_filters=['test1']
+    )
     admin.add_view(view)
 
     eq_(len(view._filters), 4)
 
-    eq_(view._filter_dict, {'Test1': [(0, 'equals'),
-                                      (1, 'not equal'),
-                                      (2, 'contains'),
-                                      (3, 'not contains')]})
+    eq_(view._filter_dict, {
+        'Test1': [
+            (0, 'equals'),
+            (1, 'not equal'),
+            (2, 'contains'),
+            (3, 'not contains')
+        ],
+    })
 
-    db.session.add(Model1('model1'))
-    db.session.add(Model1('model2'))
-    db.session.add(Model1('model3'))
-    db.session.add(Model1('model4'))
+    # Test filter that references property
+    view = CustomModelView(Model2, db.session,
+                           column_filters=['model1'])
+
+    eq_(view._filter_dict, {
+        'Model1 / Test1': [
+            (0, 'equals'),
+            (1, 'not equal'),
+            (2, 'contains'),
+            (3, 'not contains')
+        ],
+        'Model1 / Test2': [
+            (4, 'equals'),
+            (5, 'not equal'),
+            (6, 'contains'),
+            (7, 'not contains')
+        ],
+        'Model1 / Test3': [
+            (8, 'equals'),
+            (9, 'not equal'),
+            (10, 'contains'),
+            (11, 'not contains')
+        ],
+        'Model1 / Test4': [
+            (12, 'equals'),
+            (13, 'not equal'),
+            (14, 'contains'),
+            (15, 'not contains')
+        ],
+        'Model1 / Bool Field': [
+            (16, 'equals'),
+            (17, 'not equal'),
+        ],
+        'Model1 / Enum Field': [
+            (18, u'equals'),
+            (19, u'not equal'),
+        ]
+    })
+
+    # Test filter with a dot
+    view = CustomModelView(Model2, db.session,
+                           column_filters=['model1.bool_field'])
+
+    eq_(view._filter_dict, {
+        'Model1 / Bool Field': [
+            (0, 'equals'),
+            (1, 'not equal'),
+        ],
+    })
+    # Fill DB
+    model1_obj1 = Model1('model1_obj1', bool_field=True)
+    model1_obj2 = Model1('model1_obj2')
+    model1_obj3 = Model1('model1_obj3')
+    model1_obj4 = Model1('model1_obj4')
+
+    model2_obj1 = Model2('model2_obj1', model1=model1_obj1)
+    model2_obj2 = Model2('model2_obj2', model1=model1_obj1)
+    model2_obj3 = Model2('model2_obj3')
+    model2_obj4 = Model2('model2_obj4')
+    db.session.add_all([
+        model1_obj1, model1_obj2, model1_obj3, model1_obj4,
+        model2_obj1, model2_obj2, model2_obj3, model2_obj4,
+    ])
     db.session.commit()
 
     client = app.test_client()
 
-    rv = client.get('/admin/model1view/?flt0_0=model1')
+    rv = client.get('/admin/model1view/?flt0_0=model1_obj1')
     eq_(rv.status_code, 200)
-    ok_('model1' in rv.data)
-    ok_('model2' not in rv.data)
+    ok_('model1_obj1' in rv.data)
+    ok_('model1_obj2' not in rv.data)
 
-    rv = client.get('/admin/model1view/?flt0_5=model1')
+    rv = client.get('/admin/model1view/?flt0_5=model1_obj1')
     eq_(rv.status_code, 200)
-    ok_('model1' in rv.data)
-    ok_('model2' in rv.data)
+    ok_('model1_obj1' in rv.data)
+    ok_('model1_obj2' in rv.data)
 
     # Test different filter types
     view = CustomModelView(Model2, db.session,
@@ -234,6 +319,27 @@ def test_column_filters():
                                           (2, 'greater than'), (3, 'smaller than')]})
 
 
+    #Test filters to joined table field
+    view = CustomModelView(
+        Model2, db.session,
+        endpoint='_model2',
+        column_filters=['model1.bool_field'],
+        column_list=[
+            'string_field',
+            'model1.id',
+            'model1.bool_field',
+        ]
+    )
+    admin.add_view(view)
+
+    rv = client.get('/admin/_model2/?flt1_0=1')
+    eq_(rv.status_code, 200)
+    ok_('model2_obj1' in rv.data)
+    ok_('model2_obj2' in rv.data)
+    ok_('model2_obj3' not in rv.data)
+    ok_('model2_obj4' not in rv.data)
+
+
 def test_url_args():
     app, db, admin = setup()
 
@@ -379,6 +485,7 @@ def test_on_model_change_delete():
     client.post(url)
     ok_(view.deleted)
 
+
 def test_multiple_delete():
     app, db, admin = setup()
     M1, _ = create_models(db)
